/*
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: Apache-2.0
 */

package de.eshg.officialmedicalservice.procedure;

import de.eshg.base.centralfile.api.facility.GetFacilityFileStateResponse;
import de.eshg.base.centralfile.api.person.AddPersonFileStateResponse;
import de.eshg.base.centralfile.api.person.GetPersonFileStateResponse;
import de.eshg.base.user.api.UserDto;
import de.eshg.lib.auditlog.AuditLogger;
import de.eshg.lib.procedure.domain.model.PersonType;
import de.eshg.lib.procedure.domain.model.ProcedureStatus;
import de.eshg.lib.procedure.domain.model.ProcedureType;
import de.eshg.lib.procedure.domain.model.TaskStatus;
import de.eshg.lib.procedure.domain.model.TaskType;
import de.eshg.lib.procedure.mapping.ProcedureMapper;
import de.eshg.officialmedicalservice.concern.ConcernMapper;
import de.eshg.officialmedicalservice.procedure.api.EmployeeOmsProcedureOverviewDto;
import de.eshg.officialmedicalservice.procedure.api.MedicalOpinionStatusDto;
import de.eshg.officialmedicalservice.procedure.persistence.entity.MedicalOpinionResult;
import de.eshg.officialmedicalservice.procedure.persistence.entity.MedicalOpinionStatus;
import de.eshg.officialmedicalservice.procedure.persistence.entity.OmsProcedure;
import de.eshg.officialmedicalservice.procedure.persistence.entity.OmsTask;
import de.eshg.officialmedicalservice.procedure.persistence.entity.Person;
import de.eshg.officialmedicalservice.waitingroom.persistence.entity.WaitingRoom;
import java.time.Clock;
import java.time.Instant;
import java.time.LocalDate;
import java.util.UUID;
import org.springframework.stereotype.Component;

@Component
public class OmsProcedureOverviewMapper {
  private final Clock clock;
  private final AuditLogger auditLogger;

  public OmsProcedureOverviewMapper(Clock clock, AuditLogger auditLogger) {
    this.clock = clock;
    this.auditLogger = auditLogger;
  }

  public OmsProcedure toDomainType(
      UUID currentUserId, AddPersonFileStateResponse affectedPersonBaseResponse, UUID physicianId) {

    OmsProcedure procedure = new OmsProcedure();

    procedure.setProcedureType(ProcedureType.OFFICIAL_MEDICAL_SERVICE);
    procedure.updateProcedureStatus(ProcedureStatus.DRAFT, clock, auditLogger);
    procedure.setMedicalOpinionStatus(MedicalOpinionStatus.IN_PROGRESS);
    procedure.setMedicalOpinionResult(MedicalOpinionResult.UNKNOWN);
    procedure.setWaitingRoom(new WaitingRoom());

    OmsTask omsTask = new OmsTask();
    if (currentUserId != null) {
      omsTask.assign(currentUserId, currentUserId, Instant.now(clock));
    }
    omsTask.setTaskType(TaskType.OFFICIAL_MEDICAL_SERVICE);
    omsTask.setTaskStatus(TaskStatus.OPEN);
    procedure.addTask(omsTask);

    Person affectedPerson = new Person();
    affectedPerson.setCentralFileStateId(affectedPersonBaseResponse.id());
    affectedPerson.setProcedure(procedure);
    affectedPerson.setPersonType(PersonType.PATIENT);

    procedure.getRelatedPersons().add(affectedPerson);

    procedure.setPhysicianId(physicianId);

    return procedure;
  }

  public EmployeeOmsProcedureOverviewDto toInterfaceType(
      OmsProcedure procedure,
      GetPersonFileStateResponse affectedPerson,
      GetFacilityFileStateResponse facility,
      UserDto physician,
      Instant nextAppointment) {
    String firstName = null;
    String lastName = null;
    LocalDate dateOfBirth = null;
    String facilityName = null;
    String physicianName = null;
    if (affectedPerson != null) {
      firstName = affectedPerson.firstName();
      lastName = affectedPerson.lastName();
      dateOfBirth = affectedPerson.dateOfBirth();
    }
    if (facility != null) {
      facilityName = facility.name();
    }
    if (physician != null) {
      physicianName = physician.firstName() + " " + physician.lastName();
    }
    return new EmployeeOmsProcedureOverviewDto(
        procedure.getExternalId(),
        ProcedureMapper.toInterfaceType(procedure.getProcedureStatus()),
        MedicalOpinionStatusDto.valueOf(procedure.getMedicalOpinionStatus().name()),
        firstName,
        lastName,
        dateOfBirth,
        facilityName,
        ConcernMapper.mapToConcernDto(procedure.getConcern()),
        physicianName,
        nextAppointment);
  }
}
