/*
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

package de.eshg.stiprotection;

import static de.eshg.stiprotection.api.GetStiProtectionProceduresSortOrderDto.ASC;
import static de.eshg.stiprotection.pdf.identification.DocumentParameters.mapToDepartment;
import static de.eshg.stiprotection.pdf.identification.DocumentParameters.toAppointmentTimeRange;
import static de.eshg.stiprotection.pdf.identification.DocumentParameters.toConsultationAppointment;
import static de.eshg.stiprotection.pdf.identification.DocumentParameters.toDocumentDate;
import static de.eshg.stiprotection.persistence.db.StiProtectionSystemProgressEntryType.PERSON_DETAILS_UPDATED;
import static org.springframework.data.jpa.domain.Specification.allOf;

import de.eshg.base.GenderDto;
import de.eshg.base.calendar.api.TimeRange;
import de.eshg.base.citizenuser.CitizenAccessCodeUserApi;
import de.eshg.base.citizenuser.api.AddCitizenAccessCodeUserWithPinCredentialRequest;
import de.eshg.base.citizenuser.api.CitizenAccessCodeUserDto;
import de.eshg.base.citizenuser.api.CredentialTypeDto;
import de.eshg.base.citizenuser.api.VerifyCitizenAccessCodeUserCredentialsRequest;
import de.eshg.base.department.GetDepartmentInfoResponse;
import de.eshg.lib.appointmentblock.MappingUtil;
import de.eshg.lib.auditlog.AuditLogger;
import de.eshg.lib.document.generator.department.DepartmentClient;
import de.eshg.lib.document.generator.department.DepartmentLogo;
import de.eshg.lib.procedure.domain.model.Pdf;
import de.eshg.lib.procedure.domain.model.PersonType;
import de.eshg.lib.procedure.domain.model.ProcedureStatus;
import de.eshg.lib.procedure.domain.model.Procedure_;
import de.eshg.lib.procedure.domain.model.RelatedPerson;
import de.eshg.lib.procedure.domain.model.TaskStatus;
import de.eshg.lib.procedure.domain.model.TaskType;
import de.eshg.lib.procedure.model.ProcedureStatusDto;
import de.eshg.rest.service.error.BadRequestException;
import de.eshg.rest.service.security.CurrentUserHelper;
import de.eshg.stiprotection.api.ConcernDto;
import de.eshg.stiprotection.api.GetStiProtectionProceduresFilterOptions;
import de.eshg.stiprotection.api.GetStiProtectionProceduresPaginationOptions;
import de.eshg.stiprotection.api.GetStiProtectionProceduresSortByDto;
import de.eshg.stiprotection.api.GetStiProtectionProceduresSortOptions;
import de.eshg.stiprotection.api.GetStiProtectionProceduresSortOrderDto;
import de.eshg.stiprotection.api.LabStatusDto;
import de.eshg.stiprotection.api.StiProcedureOriginDto;
import de.eshg.stiprotection.department.SexWorkDepartmentInfoConfigService;
import de.eshg.stiprotection.department.StiConsultationDepartmentInfoConfigService;
import de.eshg.stiprotection.mapper.PersonMapper;
import de.eshg.stiprotection.pdf.identification.AnonymousIdentificationDocument;
import de.eshg.stiprotection.pdf.identification.AnonymousIdentificationDocumentService;
import de.eshg.stiprotection.pdf.identification.AppointmentUrls;
import de.eshg.stiprotection.pdf.identification.ConsultationAppointment;
import de.eshg.stiprotection.pdf.identification.Department;
import de.eshg.stiprotection.pdf.identification.DocumentSender;
import de.eshg.stiprotection.pdf.identification.QrCodes;
import de.eshg.stiprotection.persistence.data.PersonData;
import de.eshg.stiprotection.persistence.data.ResultPage;
import de.eshg.stiprotection.persistence.db.Concern;
import de.eshg.stiprotection.persistence.db.Gender;
import de.eshg.stiprotection.persistence.db.LabStatus;
import de.eshg.stiprotection.persistence.db.Person;
import de.eshg.stiprotection.persistence.db.Person_;
import de.eshg.stiprotection.persistence.db.StiProcedureOrigin;
import de.eshg.stiprotection.persistence.db.StiProtectionProcedure;
import de.eshg.stiprotection.persistence.db.StiProtectionProcedureRepository;
import de.eshg.stiprotection.persistence.db.StiProtectionProcedure_;
import de.eshg.stiprotection.persistence.db.StiProtectionTask;
import de.eshg.stiprotection.util.ProgressEntryUtil;
import jakarta.persistence.criteria.Path;
import jakarta.persistence.criteria.Root;
import jakarta.persistence.metamodel.SingularAttribute;
import java.time.Clock;
import java.time.Duration;
import java.time.Instant;
import java.time.LocalDate;
import java.time.Year;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.UUID;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.client.HttpClientErrorException;

@Service
public class StiProtectionProcedureService {

  private final StiProtectionProcedureRepository repository;
  private final Clock clock;
  private final AuditLogger auditLogger;
  private final AnonymousIdentificationDocumentService documentService;
  private final DepartmentClient departmentClient;
  private final StiConsultationDepartmentInfoConfigService stiConsultationDepartmentInfoService;
  private final SexWorkDepartmentInfoConfigService sexWorkDepartmentInfoService;
  private final CitizenAccessCodeUserApi citizenAccessCodeUserApi;
  private final StiProtectionProcedureFinder procedureFinder;
  private final ProgressEntryUtil progressEntryUtil;
  private final String citizenPortalUrl;

  public StiProtectionProcedureService(
      StiProtectionProcedureRepository procedures,
      Clock clock,
      AuditLogger auditLogger,
      AnonymousIdentificationDocumentService documentService,
      DepartmentClient departmentClient,
      StiConsultationDepartmentInfoConfigService stiConsultationDepartmentInfoService,
      SexWorkDepartmentInfoConfigService sexWorkDepartmentInfoService,
      CitizenAccessCodeUserApi citizenAccessCodeUserApi,
      StiProtectionProcedureFinder procedureFinder,
      ProgressEntryUtil progressEntryUtil,
      @Value("${eshg.citizen-portal.reverse-proxy.url}") String citizenPortalUrl) {
    this.repository = procedures;
    this.clock = clock;
    this.auditLogger = auditLogger;
    this.documentService = documentService;
    this.departmentClient = departmentClient;
    this.stiConsultationDepartmentInfoService = stiConsultationDepartmentInfoService;
    this.sexWorkDepartmentInfoService = sexWorkDepartmentInfoService;
    this.citizenAccessCodeUserApi = citizenAccessCodeUserApi;
    this.procedureFinder = procedureFinder;
    this.progressEntryUtil = progressEntryUtil;
    this.citizenPortalUrl = citizenPortalUrl;
  }

  public StiProtectionProcedure createProcedure(
      Concern concern, StiProcedureOrigin stiProcedureOrigin) {
    StiProtectionProcedure procedure =
        StiProtectionProcedure.newProcedure(concern, stiProcedureOrigin, clock, auditLogger);
    procedure.addTask(createTask());
    return repository.save(procedure);
  }

  public StiProtectionProcedure saveProcedure(
      Concern concern, StiProcedureOrigin stiProcedureOrigin) {
    return repository.save(
        StiProtectionProcedure.newProcedure(concern, stiProcedureOrigin, clock, auditLogger));
  }

  public void addPerson(StiProtectionProcedure procedure, PersonData personData) {
    Person person = PersonMapper.toDatabaseType(personData);
    person.setCentralFileStateId(createUniqueDummyCentralFileStateId());
    person.setPersonType(PersonType.PATIENT);
    if (procedure.getRelatedPersons().isEmpty()) {
      procedure.addRelatedPerson(person);
    } else {
      procedure.getPerson().copyFrom(person);
    }
  }

  /**
   * @deprecated The STI module does not use the central file API for storing person related
   *     information. All person information is supposed to be anonymous and will be kept directly
   *     inside the module db. Currently, every {@link RelatedPerson RelatedPerson} is
   *     <b>required</b> to have a <u>unique</u> <b>CentralFileStateId</b>. In order to fulfill this
   *     requirement we insert a random UUID as a dummy ID. <br>
   *     <br>
   *     This usage should be refactored as soon as the procedure library is flexible enough to
   *     reflect our use case.
   */
  @Deprecated(since = "forever")
  private UUID createUniqueDummyCentralFileStateId() {
    return UUID.randomUUID();
  }

  private StiProtectionTask createTask() {
    StiProtectionTask task = new StiProtectionTask();
    task.setTaskType(TaskType.STI_PROTECTION);
    task.setTaskStatus(TaskStatus.OPEN);
    task.assign(
        CurrentUserHelper.getCurrentUserId(),
        CurrentUserHelper.getCurrentUserId(),
        Instant.now(clock));
    return task;
  }

  public ResultPage<StiProtectionProcedure> getProcedures(
      GetStiProtectionProceduresSortOptions sortOptions,
      GetStiProtectionProceduresPaginationOptions paginationOptions,
      GetStiProtectionProceduresFilterOptions filterOptions) {

    PageRequest pageRequest =
        PageRequest.of(paginationOptions.pageNumber(), paginationOptions.pageSize());

    Specification<StiProtectionProcedure> spec =
        allOf(
            filterByCreatedAt(filterOptions),
            filterByYearOfBirth(filterOptions),
            filterByAppointmentDate(filterOptions),
            filterByGender(filterOptions),
            filterByConcern(filterOptions),
            filterByProcedureStatus(filterOptions),
            filterByLabStatus(filterOptions),
            filterByStiProcedureOrigin(filterOptions),
            orderBy(sortOptions.sortOrder(), sortOptions.sortBy()));

    Page<StiProtectionProcedure> procedures = repository.findAll(spec, pageRequest);
    if (procedures.isEmpty()) {
      return new ResultPage<>(0, 0, List.of());
    }

    return new ResultPage<>(
        procedures.getTotalPages(), procedures.getTotalElements(), procedures.stream().toList());
  }

  private Specification<StiProtectionProcedure> filterByStiProcedureOrigin(
      GetStiProtectionProceduresFilterOptions filterOptions) {
    Set<StiProcedureOriginDto> dto = filterOptions.procedureOrigin();
    if (dto != null) {
      return (root, query, criteriaBuilder) ->
          root.get(StiProtectionProcedure_.STI_PROCEDURE_ORIGIN)
              .in(mapFilterBy(StiProcedureOrigin.class, dto));
    } else {
      return (root, query, criteriaBuilder) -> criteriaBuilder.conjunction();
    }
  }

  private static <T extends Enum<T>, S extends Enum<S>> List<S> mapFilterBy(
      Class<S> targetEnum, Collection<T> sourceValues) {
    return sourceValues.stream()
        .filter(Objects::nonNull)
        .map(val -> MappingUtil.mapEnum(targetEnum, val))
        .toList();
  }

  private Specification<StiProtectionProcedure> filterByLabStatus(
      GetStiProtectionProceduresFilterOptions filterOptions) {
    Set<LabStatusDto> dto = filterOptions.labStatus();
    if (dto != null) {
      return (root, query, criteriaBuilder) ->
          root.get(StiProtectionProcedure_.LAB_STATUS).in(mapFilterBy(LabStatus.class, dto));
    } else {
      return (root, query, criteriaBuilder) -> criteriaBuilder.conjunction();
    }
  }

  private Specification<StiProtectionProcedure> filterByProcedureStatus(
      GetStiProtectionProceduresFilterOptions filterOptions) {
    Set<ProcedureStatusDto> dto = filterOptions.procedureStatus();
    if (dto != null) {
      return (root, query, criteriaBuilder) ->
          root.get(Procedure_.PROCEDURE_STATUS).in(mapFilterBy(ProcedureStatus.class, dto));
    } else {
      return (root, query, criteriaBuilder) -> criteriaBuilder.conjunction();
    }
  }

  private Specification<StiProtectionProcedure> filterByConcern(
      GetStiProtectionProceduresFilterOptions filterOptions) {
    Set<ConcernDto> dto = filterOptions.concern();
    if (dto != null) {
      return (root, query, criteriaBuilder) ->
          root.get(StiProtectionProcedure_.CONCERN).in(mapFilterBy(Concern.class, dto));
    } else {
      return (root, query, criteriaBuilder) -> criteriaBuilder.conjunction();
    }
  }

  private Specification<StiProtectionProcedure> filterByGender(
      GetStiProtectionProceduresFilterOptions filterOptions) {
    Set<GenderDto> dto = filterOptions.gender();
    if (dto != null) {
      return (root, query, criteriaBuilder) ->
          root.join(Procedure_.relatedPersons)
              .get(Person_.GENDER)
              .in(mapFilterBy(Gender.class, dto));
    } else {
      return (root, query, criteriaBuilder) -> criteriaBuilder.conjunction();
    }
  }

  private Specification<StiProtectionProcedure> filterByAppointmentDate(
      GetStiProtectionProceduresFilterOptions filterOptions) {
    Instant start = atStartOfDay(filterOptions.appointmentDateStart());
    Instant end = atEndOfDay(filterOptions.appointmentDateEnd());
    SingularAttribute<StiProtectionProcedure, Instant> appointmentStart =
        StiProtectionProcedure_.appointmentStart;
    if (start != null && end != null) {
      return (root, query, criteriaBuilder) ->
          criteriaBuilder.between(root.get(appointmentStart), start, end);
    } else if (start != null) {
      return (root, query, criteriaBuilder) ->
          criteriaBuilder.greaterThanOrEqualTo(root.get(appointmentStart), start);
    } else if (end != null) {
      return (root, query, criteriaBuilder) ->
          criteriaBuilder.lessThanOrEqualTo(root.get(appointmentStart), end);
    } else {
      return (root, query, criteriaBuilder) -> criteriaBuilder.conjunction();
    }
  }

  private Specification<StiProtectionProcedure> filterByYearOfBirth(
      GetStiProtectionProceduresFilterOptions filterOptions) {
    Year yearOfBirth = filterOptions.yearOfBirth();
    if (yearOfBirth != null) {
      return (root, query, criteriaBuilder) ->
          criteriaBuilder.equal(
              root.join(Procedure_.relatedPersons).get(Person_.YEAR_OF_BIRTH), yearOfBirth);
    } else {
      return (root, query, criteriaBuilder) -> criteriaBuilder.conjunction();
    }
  }

  private Specification<StiProtectionProcedure> filterByCreatedAt(
      GetStiProtectionProceduresFilterOptions filterOptions) {
    Instant start = atStartOfDay(filterOptions.creationDateStart());
    Instant end = atEndOfDay(filterOptions.creationDateEnd());
    if (start != null && end != null) {
      return (root, query, criteriaBuilder) ->
          criteriaBuilder.between(root.get(Procedure_.createdAt), start, end);
    } else if (start != null) {
      return (root, query, criteriaBuilder) ->
          criteriaBuilder.greaterThanOrEqualTo(root.get(Procedure_.createdAt), start);
    } else if (end != null) {
      return (root, query, criteriaBuilder) ->
          criteriaBuilder.lessThanOrEqualTo(root.get(Procedure_.createdAt), end);
    } else {
      return (root, query, criteriaBuilder) -> criteriaBuilder.conjunction();
    }
  }

  private Instant atStartOfDay(LocalDate date) {
    if (date == null) {
      return null;
    }
    return date.atStartOfDay(clock.getZone()).toInstant();
  }

  private Instant atEndOfDay(LocalDate date) {
    if (date == null) {
      return null;
    }
    return atStartOfDay(date).plus(Duration.ofDays(1)).minusSeconds(1);
  }

  private Specification<StiProtectionProcedure> orderBy(
      GetStiProtectionProceduresSortOrderDto sortOrder,
      GetStiProtectionProceduresSortByDto sortBy) {
    return (root, query, criteriaBuilder) -> {
      Path<?> sortProperty = getSortProperty(sortBy, root);

      if (sortOrder == ASC) {
        query.orderBy(criteriaBuilder.asc(sortProperty));
      } else {
        query.orderBy(criteriaBuilder.desc(sortProperty));
      }
      return criteriaBuilder.conjunction();
    };
  }

  private static Path<?> getSortProperty(
      GetStiProtectionProceduresSortByDto sortBy, Root<StiProtectionProcedure> root) {
    return switch (sortBy) {
      case CREATED_AT -> root.get(Procedure_.createdAt);
      case SAMPLE_BARCODE -> root.get(StiProtectionProcedure_.sampleBarCode);
      case APPOINTMENT -> root.get(StiProtectionProcedure_.appointmentStart);
    };
  }

  public void updatePersonDetails(UUID procedureId, PersonData personData) {
    StiProtectionProcedure procedure = procedureFinder.findByExternalId(procedureId);
    Person person = procedure.getPerson();
    person.setGender(personData.gender());
    person.setYearOfBirth(personData.yearOfBirth());
    person.setCountryOfBirth(personData.countryOfBirth());
    person.setInGermanySince(personData.inGermanySince());
    progressEntryUtil.addProgressEntry(procedureId, PERSON_DETAILS_UPDATED);
  }

  public void closeProcedure(StiProtectionProcedure procedure) {
    ProcedureStatus procedureStatus = procedure.getProcedureStatus();
    if (procedureStatus.isOpen()) {
      procedure.updateProcedureStatus(ProcedureStatus.CLOSED, clock, auditLogger);
    } else {
      throw unexpectedProcedureStatus(procedure.getExternalId(), procedureStatus);
    }
  }

  public void reopenProcedure(UUID procedureId) {
    StiProtectionProcedure procedure = procedureFinder.findByExternalId(procedureId);
    ProcedureStatus procedureStatus = procedure.getProcedureStatus();
    if (!procedureStatus.isOpen()) {
      procedure.updateProcedureStatus(ProcedureStatus.OPEN, clock, auditLogger);
    } else {
      throw unexpectedProcedureStatus(procedureId, procedureStatus);
    }
  }

  protected static BadRequestException unexpectedProcedureStatus(
      UUID procedureId, ProcedureStatus procedureStatus) {
    return new BadRequestException(
        "%s: unexpected procedure status: %s".formatted(procedureId, procedureStatus));
  }

  public Pdf getAnonymousIdentificationDocument(UUID procedureId) {
    StiProtectionProcedure procedure = procedureFinder.findByExternalId(procedureId);
    TimeRange timeRange = toAppointmentTimeRange(procedure);
    Department department = getRelavantDepartmentInfo(procedure);
    String documentDate = toDocumentDate(clock.instant());
    DepartmentLogo departmentLogo = departmentClient.getDepartmentLogo();
    String accessCode = getAccessCode(procedure);
    String qrCode = QrCodes.qrCode(citizenPortalUrl, procedure.getConcern(), accessCode);
    String appointmentUrl = AppointmentUrls.url(citizenPortalUrl, procedure.getConcern());
    ConsultationAppointment appointment =
        toConsultationAppointment(department, timeRange, appointmentUrl, accessCode, qrCode);
    DocumentSender sender = new DocumentSender(department, documentDate, departmentLogo);
    return documentService.createPdf(new AnonymousIdentificationDocument(sender, appointment));
  }

  private Department getRelavantDepartmentInfo(StiProtectionProcedure procedure) {
    GetDepartmentInfoResponse departmentInfoResponse =
        switch (procedure.getConcern()) {
          case HIV_STI_CONSULTATION -> stiConsultationDepartmentInfoService.getDepartmentInfo();
          case SEX_WORK -> sexWorkDepartmentInfoService.getDepartmentInfo();
        };

    return mapToDepartment(departmentInfoResponse);
  }

  private String getAccessCode(StiProtectionProcedure procedure) {
    UUID anonymousUserId = procedure.getAnonymousUserId();
    if (anonymousUserId == null) {
      throw new BadRequestException("Anonymous user not registered");
    }
    String accessCode = procedure.getAccessCode();
    if (!StringUtils.hasText(accessCode)) {
      throw new BadRequestException("Access code cannot be null or blank");
    }
    return accessCode;
  }

  public String generatePin() {
    return RandomStringUtils.secure().nextNumeric(6);
  }

  public void registerAnonymousUser(StiProtectionProcedure procedure, String pin) {
    UUID anonymousUserId = procedure.getAnonymousUserId();
    if (anonymousUserId != null) {
      throw new BadRequestException("User already registered.");
    }
    CitizenAccessCodeUserDto user =
        citizenAccessCodeUserApi.addCitizenAccessCodeUserWithPinCredential(
            new AddCitizenAccessCodeUserWithPinCredentialRequest(pin));
    procedure.setAnonymousUserId(user.userId());
    procedure.setAccessCode(user.accessCode());
  }

  public void deleteAnonymousUser(StiProtectionProcedure procedure) {
    UUID anonymousUserId = procedure.getAnonymousUserId();
    if (anonymousUserId != null) {
      citizenAccessCodeUserApi.deleteCitizenAccessCodeUser(anonymousUserId);
      procedure.setAnonymousUserId(null);
      procedure.setAccessCode(null);
    }
  }

  public void verifyAnonymousUserPin(UUID procedureId, String pin) {
    StiProtectionProcedure procedure = procedureFinder.findByExternalId(procedureId);
    try {
      UUID userId =
          Optional.ofNullable(procedure.getAnonymousUserId())
              .orElseThrow(() -> new BadRequestException("Procedure has no user"));
      citizenAccessCodeUserApi.verifyCitizenAccessCodeUserCredentials(
          userId, new VerifyCitizenAccessCodeUserCredentialsRequest(CredentialTypeDto.PIN, pin));
    } catch (HttpClientErrorException.BadRequest e) {
      throw new BadRequestException("Invalid credentials");
    }
  }

  public StiProtectionProcedure findByExternalId(UUID procedureId) {
    return procedureFinder.findByExternalId(procedureId);
  }

  public void deleteProcedure(StiProtectionProcedure procedure) {
    repository.delete(procedure);
  }

  public List<StiProtectionProcedure> findProcedures(String text) {
    return procedureFinder.findProcedures(text);
  }
}
