/**
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

import {
  ExaminationResult,
  ExaminationStatus,
  FluoridationExaminationResult,
  ScreeningExaminationResult,
} from "@eshg/dental";
import { isEmptyString } from "@eshg/lib-portal/helpers/guards";
import { isEmptyArray } from "formik";

export function canBeMarkedAbsent(
  status: ExaminationStatus,
  result: ExaminationResult | undefined,
): boolean {
  if (result === undefined) {
    return true;
  }
  if (status !== "OPEN") {
    return false;
  }

  switch (result.type) {
    case "screening":
      return screeningExaminationResultEmpty(
        result,
        screeningExaminationResultEmptinessRules,
      );
    case "fluoridation":
      return fluoridationExaminationResultEmpty(
        result,
        fluoridationResultEmptinessRules,
      );
    case "absence":
      return false;
  }
}

export type EmptinessRules<T> = {
  [K in keyof T]-?: (value: T[K]) => boolean;
};

const screeningExaminationResultEmptinessRules: EmptinessRules<ScreeningExaminationResult> =
  {
    type: () => true,
    dentitionType: () => true,
    oralHygieneStatus: (value) => value === undefined,
    mihStatus: (value) => value === undefined,
    orthodonticStatus: (value) => value === undefined,
    orthodonticFindings: isEmptyArray,
    fluorideVarnishApplied: (value) => isEmptyString(value) || !value,
    plaque: (value) => !value,
    calculus: (value) => !value,
    gingivitis: (value) => !value,
    parodontitis: (value) => !value,
    toothDiagnoses: (value) =>
      Object.values(value).every(
        (tooth) =>
          tooth.mainResult === undefined &&
          tooth.secondaryResult1 === undefined &&
          tooth.secondaryResult2 === undefined,
      ),
  };

function screeningExaminationResultEmpty<T extends ScreeningExaminationResult>(
  screeningExaminationResult: T,
  emptinessChecks: { [K in keyof T]: (val: T[K]) => boolean },
): boolean {
  const keys = Object.keys(screeningExaminationResult) as (keyof T)[];
  return keys.every((key) => {
    const isEmpty = emptinessChecks[key];
    const value = screeningExaminationResult[key];
    return isEmpty(value);
  });
}

const fluoridationResultEmptinessRules: EmptinessRules<FluoridationExaminationResult> =
  {
    type: () => true,
    fluorideVarnishApplied: (value) => value === undefined || !value,
  };

function fluoridationExaminationResultEmpty<
  T extends FluoridationExaminationResult,
>(
  fluoridationExaminationResult: T,
  emptinessChecks: { [K in keyof T]: (val: T[K]) => boolean },
): boolean {
  const keys = Object.keys(fluoridationExaminationResult) as (keyof T)[];
  return keys.every((key) => {
    const isEmpty = emptinessChecks[key];
    const value = fluoridationExaminationResult[key];
    return isEmpty(value);
  });
}
