/**
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

"use client";

import {
  ButtonBar,
  SidebarActions,
  SidebarContent,
} from "@eshg/lib-employee-portal";
import { downloadFileAndOpen } from "@eshg/lib-portal/api/files/download";
import { RequiresChildren } from "@eshg/lib-portal/types/react";
import { ApiImportStatistics } from "@eshg/medical-registry-api";
import {
  ErrorOutlineOutlined,
  FileDownloadOutlined,
  InfoOutlined,
  WarningAmberOutlined,
} from "@mui/icons-material";
import {
  Button,
  List,
  ListItem,
  ListItemDecorator,
  Sheet,
  Stack,
  Typography,
} from "@mui/joy";

interface ImportDataSummarySidebarProps {
  onClose: () => void;
  onReset: () => void;
  file: File;
  statistics: ApiImportStatistics;
}

export function ImportDataSummarySidebar({
  file,
  statistics,
  onClose,
  onReset,
}: ImportDataSummarySidebarProps) {
  const requiresTroubleshooting =
    statistics.duplicated > 0 || statistics.failed > 0;
  return (
    <>
      <SidebarContent title="Daten importieren">
        <Stack gap={3}>
          <Typography level="title-md" color="success">
            Import erfolgreich
          </Typography>
          <StatisticsDisplay statistics={statistics} file={file} />
        </Stack>
      </SidebarContent>
      <SidebarActions>
        <ButtonBar
          right={
            requiresTroubleshooting ? (
              <Button onClick={onReset}>
                Liste zur Fehlerbehebung hochladen
              </Button>
            ) : (
              <Button onClick={onClose}>Fertig</Button>
            )
          }
        />
      </SidebarActions>
    </>
  );
}

function StatisticsDisplay({
  statistics,
  file,
}: {
  statistics: ApiImportStatistics;
  file: File;
}) {
  const { created, total, duplicated, failed } = statistics;
  const requiresTroubleshooting = duplicated > 0 || failed > 0;

  return (
    <>
      <Sheet color="neutral" variant="soft">
        <List
          aria-label="Statistiken"
          sx={(theme) => ({
            "--List-padding": 0,
            "--List-gap": theme.spacing(3),
            "--ListItem-paddingX": theme.spacing(2),
            "--ListItem-paddingY": theme.spacing(2),
          })}
        >
          <ListItem>
            <ListItemDecorator>
              <InfoOutlined color="primary" size="sm" />
            </ListItemDecorator>
            <Typography fontWeight="lg">
              {total === 1
                ? `${created} von ${total} Datensatz angelegt`
                : `${created} von ${total} Datensätzen angelegt`}
            </Typography>
          </ListItem>
          {requiresTroubleshooting && (
            <>
              <ListItem>
                <ListItemDecorator>
                  <WarningAmberOutlined color="warning" size="sm" />
                </ListItemDecorator>
                <Typography fontWeight="lg">
                  {duplicated === 1
                    ? `${duplicated} doppelter Datensatz`
                    : `${duplicated} doppelte Datensätze`}
                </Typography>
              </ListItem>
              <ListItem>
                <ListItemDecorator>
                  <ErrorOutlineOutlined color="danger" size="sm" />
                </ListItemDecorator>
                <Typography fontWeight="lg">
                  {failed === 1
                    ? `${failed} fehlerhafter Datensatz`
                    : `${failed} fehlerhafte Datensätze`}
                </Typography>
              </ListItem>
            </>
          )}
        </List>
      </Sheet>
      <Stack gap={1}>
        {requiresTroubleshooting ? (
          <Typography>
            Eine Liste zur Fehlerbehebung wurde automatisch heruntergeladen.
            Korrigieren Sie die in der Datei markierten Fehler und laden Sie die
            Liste erneut hoch. Bereits übertragene Daten sind markiert und
            werden nicht als Duplikat angelegt.
          </Typography>
        ) : (
          <Typography>
            Eine Liste zur Übersicht der übermittelten Daten wurde automatisch
            heruntergeladen.
          </Typography>
        )}
        <DownloadFileButton file={file}>
          {requiresTroubleshooting
            ? "Liste zur Fehlerbehebung erneut herunterladen"
            : "Liste erneut herunterladen"}
        </DownloadFileButton>
      </Stack>
    </>
  );
}

function DownloadFileButton({
  file,
  children,
}: RequiresChildren & { file: File }) {
  function download() {
    downloadFileAndOpen(file);
  }

  return (
    <Button
      variant="soft"
      color="warning"
      startDecorator={<FileDownloadOutlined />}
      onClick={() => download()}
      sx={{ justifyContent: "flex-start" }}
    >
      {children}
    </Button>
  );
}
