/**
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

import {
  OverlayBoundary,
  ProcedureLabel,
  ProcedureLabelAutocomplete,
} from "@eshg/lib-employee-portal";
import { SelectOptions } from "@eshg/lib-portal/components/formFields/SelectOptions";
import {
  isDateString,
  toDateString,
  toUtcDate,
} from "@eshg/lib-portal/helpers/dateTime";
import { GetProceduresRequest } from "@eshg/school-entry-api";
import { FormControl, FormLabel, Input } from "@mui/joy";
import { isEmpty } from "remeda";

import { useLabelApi } from "@/lib/businessModules/schoolEntry/api/clients";
import { schoolEntryApiQueryKey } from "@/lib/businessModules/schoolEntry/api/queries/apiQueryKeys";
import { PROCEDURE_TYPE_OPTIONS } from "@/lib/businessModules/schoolEntry/features/procedures/options";
import { ResettableSingleSelect } from "@/lib/shared/components/ResettableSingleSelect";
import { ActiveFilter } from "@/lib/shared/components/filterSettings/ActiveFilter";
import { FilterSettingsContent } from "@/lib/shared/components/filterSettings/FilterSettingsContent";
import {
  FilterSettingsSheet,
  FilterSettingsSheetProps,
} from "@/lib/shared/components/filterSettings/FilterSettingsSheet";
import { SearchInstitutionFilter } from "@/lib/shared/components/filterSettings/SearchInstitutionFilter";
import { SetDictionaryFilterFn } from "@/lib/shared/components/filterSettings/useFilterDictionary";
import { SchoolYearAutocomplete } from "@/lib/shared/components/formFields/schoolYear";

export type ProcedureFilters = Pick<
  GetProceduresRequest,
  | "procedureTypeFilter"
  | "schoolIdFilter"
  | "dayOfAppointmentFilter"
  | "hasAppointmentFilter"
  | "schoolYearFilter"
  | "isInvitationSentFilter"
> & { labelsFilter?: ProcedureLabel[] };

const FILTER_NAMES: Record<keyof ProcedureFilters, string> = {
  procedureTypeFilter: "Art",
  schoolIdFilter: "Schule",
  dayOfAppointmentFilter: "Untersuchung am",
  hasAppointmentFilter: "Termin",
  schoolYearFilter: "Schuljahr",
  labelsFilter: "Kennungen",
  isInvitationSentFilter: "Einladung versandt",
};

function getFilterLabel(filterValue: ActiveFilter<keyof ProcedureFilters>) {
  return FILTER_NAMES[filterValue.key];
}

interface ProcedureFilterSettingsProps {
  filterFormValues: ProcedureFilters;
  setFilterFormValue: SetDictionaryFilterFn<
    keyof ProcedureFilters,
    ProcedureFilters
  >;
  deleteFilterValue: (key: keyof ProcedureFilters) => void;
  clearFilterValues: () => void;
  filterSettingsSheetProps: FilterSettingsSheetProps;
  activeFilters: ActiveFilter<keyof ProcedureFilters>[];
}

function evaluateBooleanValue(value: boolean | undefined) {
  return value === true ? "true" : value === false ? "false" : "";
}

function evaluateStringAsBoolean(value: string) {
  return value === "true" ? true : value === "false" ? false : undefined;
}

export function ProcedureFilterSettings(props: ProcedureFilterSettingsProps) {
  const labelApi = useLabelApi();

  return (
    <OverlayBoundary>
      <FilterSettingsSheet {...props.filterSettingsSheetProps}>
        <FilterSettingsContent
          showActiveFilters={props.activeFilters.length > 0}
          activeFilters={
            <ActiveFilter
              maxVisible={5}
              filterValues={props.activeFilters}
              deleteAllFilterValues={props.clearFilterValues}
              deleteFilterValue={props.deleteFilterValue}
              getFilterValueLabel={getFilterLabel}
            />
          }
        >
          <FormControl>
            <FormLabel>Untersuchung am</FormLabel>
            <Input
              type="date"
              value={
                props.filterFormValues.dayOfAppointmentFilter !== undefined
                  ? toDateString(props.filterFormValues.dayOfAppointmentFilter)
                  : ""
              }
              onChange={(dayOfAppointment) => {
                const value = dayOfAppointment.target.value;
                props.setFilterFormValue(
                  "dayOfAppointmentFilter",
                  isDateString(value) ? toUtcDate(value) : undefined,
                );
              }}
            />
          </FormControl>
          <FormControl>
            <FormLabel>Termin</FormLabel>
            <ResettableSingleSelect
              value={evaluateBooleanValue(
                props.filterFormValues.hasAppointmentFilter,
              )}
              onChange={(_, newValue) => {
                if (newValue === null) {
                  return;
                }
                props.setFilterFormValue(
                  "hasAppointmentFilter",
                  evaluateStringAsBoolean(newValue),
                );
              }}
              onResetSelect={() => {
                props.setFilterFormValue("hasAppointmentFilter", undefined);
              }}
            >
              <SelectOptions
                options={[
                  { value: "true", label: "mit Termin" },
                  { value: "false", label: "ohne Termin" },
                ]}
              />
            </ResettableSingleSelect>
          </FormControl>
          <FormControl>
            <FormLabel>Einladung versandt</FormLabel>
            <ResettableSingleSelect
              value={evaluateBooleanValue(
                props.filterFormValues.isInvitationSentFilter,
              )}
              onChange={(_, newValue) => {
                if (newValue === null) {
                  return;
                }
                props.setFilterFormValue(
                  "isInvitationSentFilter",
                  evaluateStringAsBoolean(newValue),
                );
              }}
              onResetSelect={() => {
                props.setFilterFormValue("isInvitationSentFilter", undefined);
              }}
            >
              <SelectOptions
                options={[
                  { value: "true", label: "Ja" },
                  { value: "false", label: "Nein" },
                ]}
              />
            </ResettableSingleSelect>
          </FormControl>
          <FormControl>
            <FormLabel>Schuljahr</FormLabel>
            <SchoolYearAutocomplete
              value={props.filterFormValues.schoolYearFilter ?? null}
              onChange={(_, newValue) => {
                props.setFilterFormValue(
                  "schoolYearFilter",
                  newValue ?? undefined,
                );
              }}
            />
          </FormControl>
          <FormControl>
            <FormLabel>Schule</FormLabel>
            <SearchInstitutionFilter
              institutionId={props.filterFormValues.schoolIdFilter}
              onChange={(schoolId) =>
                props.setFilterFormValue("schoolIdFilter", schoolId)
              }
              placeholder="Schule suchen"
            />
          </FormControl>
          <FormControl>
            <FormLabel>Art</FormLabel>
            <ResettableSingleSelect
              aria-label="Art"
              value={props.filterFormValues.procedureTypeFilter ?? ""}
              onChange={(_, newValue) => {
                if (newValue === null) {
                  return;
                }
                props.setFilterFormValue("procedureTypeFilter", newValue);
              }}
              onResetSelect={() => {
                props.setFilterFormValue("procedureTypeFilter", undefined);
              }}
            >
              <SelectOptions options={PROCEDURE_TYPE_OPTIONS} />
            </ResettableSingleSelect>
          </FormControl>
          <FormControl>
            <FormLabel>Kennungen</FormLabel>
            <ProcedureLabelAutocomplete
              name="labels"
              value={props.filterFormValues.labelsFilter ?? []}
              onChange={(newValue) => {
                props.setFilterFormValue(
                  "labelsFilter",
                  isEmpty(newValue) ? undefined : newValue,
                );
              }}
              procedureLabelApi={labelApi}
              procedureLabelApiQueryKey={schoolEntryApiQueryKey}
            />
          </FormControl>
        </FilterSettingsContent>
      </FilterSettingsSheet>
    </OverlayBoundary>
  );
}
